// 2007/08/03 Hiroyuki Ogasawara
// vim:ts=4 sw=4:
/*!
	@file
	@date	2007/08/03
	@author	Hiroyuki Ogasawara
	@brief

	obt@ Cube Őĕ`悵Ă܂VF[_[

	@li http://flatlib.main.jp/dench/
	@li http://ch09144.kitaguni.tv/
*/


#include	<windows.h>
#include	<d3d10.h>
#include	<d3dx10.h>

#define		ZRELEASE( inf )		((inf)?((inf)->Release(),(inf)=NULL):0)


#define		_APP_TITLE_		TEXT("Wheel Handle ss00")

enum {
	WINDOW_SIZE_WIDTH	= 1024,
	WINDOW_SIZE_HEIGHT	= 768,
};


static IDXGISwapChain*			g_iSwapChain= NULL;
static ID3D10Device*			g_iDevice= NULL;
static ID3D10RenderTargetView*	g_iRenderTargetView= NULL;
static ID3D10DepthStencilView*	g_iDepthStencilView= NULL;
static ID3D10Effect*			g_iEffect= NULL;
static float					g_Angle= 0.0f;


/*!
	@param[in]	hwnd
	@param[in]	winmode
	@param[in]	width
	@param[in]	height
	@brief
		Direct3D10 ̎蔲

	R[hZ邱ƍŗDŃG[`FbN܂B
*/
static void
InitDevice( HWND hwnd, int winmode, int width, int height )
{
	DXGI_SWAP_CHAIN_DESC	swapdesc= {
		{
			width, height,
			{	60,	1,	},
			DXGI_FORMAT_R8G8B8A8_UNORM,
			DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED,
			DXGI_MODE_SCALING_UNSPECIFIED,
		},
		{	1,	0,	},	// sample
		DXGI_USAGE_RENDER_TARGET_OUTPUT,
		1,
		hwnd,
		winmode,
		DXGI_SWAP_EFFECT_DISCARD,
		0
	};

	D3D10CreateDeviceAndSwapChain(
			NULL,
			D3D10_DRIVER_TYPE_HARDWARE,
			NULL,
			D3D10_CREATE_DEVICE_DEBUG,
			D3D10_SDK_VERSION,
			&swapdesc,
			&g_iSwapChain,
			&g_iDevice
		);

	ID3D10Texture2D*	iBackBuffer= NULL;
	g_iSwapChain->GetBuffer( 0, __uuidof( ID3D10Texture2D ),
							reinterpret_cast<void**>( &iBackBuffer ) );
	g_iDevice->CreateRenderTargetView( iBackBuffer, NULL, &g_iRenderTargetView );
	ZRELEASE( iBackBuffer );

	ID3D10Texture2D*	iTexture= NULL;
	D3D10_TEXTURE2D_DESC	tex2ddesc= {
		width, height, 1, 1,	// mip, array
		DXGI_FORMAT_D32_FLOAT,
		{	1,	0,	},	// sample
		D3D10_USAGE_DEFAULT,
		D3D10_BIND_DEPTH_STENCIL,
		0,	// cpu
		0	// misc
	};
	g_iDevice->CreateTexture2D( &tex2ddesc, NULL, &iTexture );

	D3D10_DEPTH_STENCIL_VIEW_DESC	vdesc= {
		DXGI_FORMAT_D32_FLOAT,
		D3D10_DSV_DIMENSION_TEXTURE2D
	};
	g_iDevice->CreateDepthStencilView( iTexture, &vdesc, &g_iDepthStencilView );
	ZRELEASE( iTexture );

	g_iDevice->OMSetRenderTargets( 1,
						&g_iRenderTargetView, g_iDepthStencilView );

	D3D10_VIEWPORT	viewport= {
		0, 0, width, height, 0.0f, 1.0f
	};
	g_iDevice->RSSetViewports( 1, &viewport );


	ID3D10Blob*	iblob= NULL;
	if( FAILED( D3DX10CreateEffectFromFile(
				TEXT("cube.fx"),
				NULL, NULL, "fx_4_0", 0, 0, g_iDevice,
				NULL, NULL, &g_iEffect, &iblob, NULL ) ) ){
		OutputDebugStringA( reinterpret_cast<char*>(
							iblob->GetBufferPointer() ) );
		ZRELEASE( iblob );
	}

	D3DXMATRIX	projection;
	D3DXMatrixPerspectiveFovLH( &projection,
			3.141592f/3.0f,
			WINDOW_SIZE_WIDTH/(float)WINDOW_SIZE_HEIGHT,
			1.0f, 10000.0f );
	g_iEffect->GetVariableByName( "Projection" )->AsMatrix()->SetMatrix(
							(float*)&projection );
}


static void
ReleaseDevice()
{
	ZRELEASE( g_iRenderTargetView );
	ZRELEASE( g_iDepthStencilView );
	ZRELEASE( g_iDevice );
	ZRELEASE( g_iSwapChain );
}


/*!
	@brief
		`惋[v
*/
static void
Render()
{
	float	color[4]= {
		0.0f, 0.0f, 1.0f, 0.0f
	};
	g_iDevice->ClearRenderTargetView( g_iRenderTargetView, color );
	g_iDevice->ClearDepthStencilView( g_iDepthStencilView,
						D3D10_CLEAR_DEPTH|D3D10_CLEAR_STENCIL, 1.0f, 0 );
	g_Angle+= 3.141592f/180.0f * 0.2f;

	D3DXMATRIX	view;
	D3DXMATRIX	matrix;
	D3DXMatrixRotationY( &view, g_Angle );
	D3DXMatrixTranslation( &matrix, 0.0f, 0.0f, 12.0f );
	D3DXMatrixMultiply( &view, &view, &matrix );
	g_iEffect->GetVariableByName( "WorldToView" )->AsMatrix()->SetMatrix(
							(float*)&view );
	g_iEffect->GetTechniqueByIndex( 0 )->GetPassByIndex( 0 )->Apply( 0 );
	g_iDevice->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );
	g_iDevice->DrawInstanced( 36, 12, 0, 0 );

	g_iSwapChain->Present( 0, 0 );
}



LRESULT CALLBACK
WindowExec( HWND hwnd, UINT mes, WPARAM wparam, LPARAM lparam )
{
	switch( mes ){
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return	FALSE;
	}
	return	DefWindowProc( hwnd, mes, wparam, lparam );
}


/*!
	@brief
		蔲main
*/
int APIENTRY
WinMain( HINSTANCE hinstance, HINSTANCE hpinstance, LPSTR cmdline, int show )
{

	WNDCLASS	wc;
	wc.style= CS_HREDRAW|CS_VREDRAW;
	wc.lpfnWndProc= WindowExec;
	wc.cbClsExtra= 0;
	wc.cbWndExtra= 0;
	wc.hInstance= hinstance;
	wc.hIcon= NULL;
	wc.hCursor= LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground= (HBRUSH)GetStockObject( BLACK_BRUSH );
	wc.lpszMenuName= NULL;
	wc.lpszClassName= _APP_TITLE_;
	RegisterClass( &wc );

	HWND	hWnd= CreateWindowEx( WS_EX_APPWINDOW,
					_APP_TITLE_, _APP_TITLE_,
					WS_OVERLAPPEDWINDOW,
					CW_USEDEFAULT, CW_USEDEFAULT,
					WINDOW_SIZE_WIDTH, WINDOW_SIZE_HEIGHT,
					NULL, NULL, hinstance, NULL );
	ShowWindow( hWnd, SW_SHOWNORMAL );
	UpdateWindow( hWnd );
	InitDevice( hWnd, TRUE, WINDOW_SIZE_WIDTH, WINDOW_SIZE_HEIGHT );

	for(;;){
		for( MSG msg ; PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ;){
			if( msg.message == WM_QUIT ){
				ReleaseDevice();
				return	0;
			}
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		Render();
		Sleep( 1 );
	}

	return	0;
}

